@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 Facter.add(:iptables_persistent_version) do
-  confine operatingsystem: ['Debian', 'Ubuntu']
+  confine 'os.name': ['Debian', 'Ubuntu']
   setcode do
     # Throw away STDERR because dpkg >= 1.16.7 will make some noise if the
     # package isn't currently installed.
